@@ -10,7 +10,7 @@
    This file is part of LibVEX, a library for dynamic binary
    instrumentation and translation.
 
-   Copyright (C) 2004-2005 OpenWorks LLP.  All rights reserved.
+   Copyright (C) 2004-2006 OpenWorks LLP.  All rights reserved.
 
    This library is made available under a dual licensing scheme.
 
@@ -703,14 +703,13 @@ X86Instr* X86Instr_Bsfr32 ( Bool isFwds, HReg src, HReg dst ) {
    i->Xin.Bsfr32.dst    = dst;
    return i;
 }
-X86Instr* X86Instr_MFence ( VexSubArch subarch )
+X86Instr* X86Instr_MFence ( UInt hwcaps )
 {
-   X86Instr* i           = LibVEX_Alloc(sizeof(X86Instr));
-   i->tag                = Xin_MFence;
-   i->Xin.MFence.subarch = subarch;
-   vassert(subarch == VexSubArchX86_sse0
-           || subarch == VexSubArchX86_sse1
-           || subarch == VexSubArchX86_sse2);
+   X86Instr* i          = LibVEX_Alloc(sizeof(X86Instr));
+   i->tag               = Xin_MFence;
+   i->Xin.MFence.hwcaps = hwcaps;
+   vassert(0 == (hwcaps & ~(VEX_HWCAPS_X86_SSE1|VEX_HWCAPS_X86_SSE2
+                                               |VEX_HWCAPS_X86_SSE3)));
    return i;
 }
 
@@ -877,7 +876,8 @@ X86Instr* X86Instr_SseShuf ( Int order, HReg src, HReg dst ) {
    return i;
 }
 
-void ppX86Instr ( X86Instr* i ) {
+void ppX86Instr ( X86Instr* i, Bool mode64 ) {
+   vassert(mode64 == False);
    switch (i->tag) {
       case Xin_Alu32R:
          vex_printf("%sl ", showX86AluOp(i->Xin.Alu32R.op));
@@ -941,14 +941,16 @@ void ppX86Instr ( X86Instr* i ) {
             vex_printf("if (%%eflags.%s) { ", 
                        showX86CondCode(i->Xin.Goto.cond));
 	 }
-         if (i->Xin.Goto.jk != Ijk_Boring) {
+         if (i->Xin.Goto.jk != Ijk_Boring
+             && i->Xin.Goto.jk != Ijk_Call
+             && i->Xin.Goto.jk != Ijk_Ret) {
             vex_printf("movl $");
             ppIRJumpKind(i->Xin.Goto.jk);
             vex_printf(",%%ebp ; ");
          }
          vex_printf("movl ");
          ppX86RI(i->Xin.Goto.dst);
-         vex_printf(",%%eax ; ret");
+         vex_printf(",%%eax ; movl $dispatcher_addr,%%edx ; jmp *%%edx");
          if (i->Xin.Goto.cond != Xcc_ALWAYS) {
             vex_printf(" }");
 	 }
@@ -985,7 +987,7 @@ void ppX86Instr ( X86Instr* i ) {
          return;
       case Xin_MFence:
          vex_printf("mfence(%s)",
-                    LibVEX_ppVexSubArch(i->Xin.MFence.subarch));
+                    LibVEX_ppVexHwCaps(VexArchX86,i->Xin.MFence.hwcaps));
          return;
       case Xin_FpUnary:
          vex_printf("g%sD ", showX86FpOp(i->Xin.FpUnary.op));
@@ -1128,9 +1130,10 @@ void ppX86Instr ( X86Instr* i ) {
 
 /* --------- Helpers for register allocation. --------- */
 
-void getRegUsage_X86Instr (HRegUsage* u, X86Instr* i)
+void getRegUsage_X86Instr (HRegUsage* u, X86Instr* i, Bool mode64)
 {
    Bool unary;
+   vassert(mode64 == False);
    initHRegUsage(u);
    switch (i->tag) {
       case Xin_Alu32R:
@@ -1214,8 +1217,13 @@ void getRegUsage_X86Instr (HRegUsage* u, X86Instr* i)
          return;
       case Xin_Goto:
          addRegUsage_X86RI(u, i->Xin.Goto.dst);
-         addHRegUse(u, HRmWrite, hregX86_EAX());
-         if (i->Xin.Goto.jk != Ijk_Boring)
+         addHRegUse(u, HRmWrite, hregX86_EAX()); /* used for next guest addr */
+         addHRegUse(u, HRmWrite, hregX86_EDX()); /* used for dispatcher addr */
+         if (i->Xin.Goto.jk != Ijk_Boring
+             && i->Xin.Goto.jk != Ijk_Call
+             && i->Xin.Goto.jk != Ijk_Ret)
+            /* note, this is irrelevant since ebp is not actually
+               available to the allocator.  But still .. */
             addHRegUse(u, HRmWrite, hregX86_EBP());
          return;
       case Xin_CMov32:
@@ -1348,19 +1356,20 @@ void getRegUsage_X86Instr (HRegUsage* u, X86Instr* i)
          addHRegUse(u, HRmWrite, i->Xin.SseShuf.dst);
          return;
       default:
-         ppX86Instr(i);
+         ppX86Instr(i, False);
          vpanic("getRegUsage_X86Instr");
    }
 }
 
 /* local helper */
-static void mapReg(HRegRemap* m, HReg* r)
+static void mapReg( HRegRemap* m, HReg* r )
 {
    *r = lookupHRegRemap(m, *r);
 }
 
-void mapRegs_X86Instr (HRegRemap* m, X86Instr* i)
+void mapRegs_X86Instr ( HRegRemap* m, X86Instr* i, Bool mode64 )
 {
+   vassert(mode64 == False);
    switch (i->tag) {
       case Xin_Alu32R:
          mapRegs_X86RMI(m, i->Xin.Alu32R.src);
@@ -1493,7 +1502,7 @@ void mapRegs_X86Instr (HRegRemap* m, X86Instr* i)
          mapReg(m, &i->Xin.SseShuf.dst);
          return;
       default:
-         ppX86Instr(i);
+         ppX86Instr(i, mode64);
          vpanic("mapRegs_X86Instr");
    }
 }
@@ -1537,11 +1546,12 @@ Bool isMove_X86Instr ( X86Instr* i, HReg* src, HReg* dst )
    register allocator.  Note it's critical these don't write the
    condition codes. */
 
-X86Instr* genSpill_X86 ( HReg rreg, Int offsetB )
+X86Instr* genSpill_X86 ( HReg rreg, Int offsetB, Bool mode64 )
 {
    X86AMode* am;
    vassert(offsetB >= 0);
    vassert(!hregIsVirtual(rreg));
+   vassert(mode64 == False);
    am = X86AMode_IR(offsetB, hregX86_EBP());
 
    switch (hregClass(rreg)) {
@@ -1557,11 +1567,12 @@ X86Instr* genSpill_X86 ( HReg rreg, Int offsetB )
    }
 }
 
-X86Instr* genReload_X86 ( HReg rreg, Int offsetB )
+X86Instr* genReload_X86 ( HReg rreg, Int offsetB, Bool mode64 )
 {
    X86AMode* am;
    vassert(offsetB >= 0);
    vassert(!hregIsVirtual(rreg));
+   vassert(mode64 == False);
    am = X86AMode_IR(offsetB, hregX86_EBP());
    switch (hregClass(rreg)) {
       case HRcInt32:
@@ -1827,7 +1838,8 @@ static UChar* push_word_from_tags ( UChar* p, UShort tags )
    Note that buf is not the insn's final place, and therefore it is
    imperative to emit position-independent code. */
 
-Int emit_X86Instr ( UChar* buf, Int nbuf, X86Instr* i )
+Int emit_X86Instr ( UChar* buf, Int nbuf, X86Instr* i, 
+                    Bool mode64, void* dispatch )
 {
    UInt irno, opc, opc_rr, subopc_imm, opc_imma, opc_cl, opc_imm, subopc;
 
@@ -1835,13 +1847,14 @@ Int emit_X86Instr ( UChar* buf, Int nbuf, X86Instr* i )
    UChar* p = &buf[0];
    UChar* ptmp;
    vassert(nbuf >= 32);
+   vassert(mode64 == False);
 
    /* Wrap an integer as a int register, for use assembling
       GrpN insns, in which the greg field is used as a sub-opcode
       and does not really contain a register. */
 #  define fake(_n) mkHReg((_n), HRcInt32, False)
 
-   /* vex_printf("asm  ");ppX86Instr(i); vex_printf("\n"); */
+   /* vex_printf("asm  ");ppX86Instr(i, mode64); vex_printf("\n"); */
 
    switch (i->tag) {
 
@@ -2147,9 +2160,9 @@ Int emit_X86Instr ( UChar* buf, Int nbuf, X86Instr* i )
          case Ijk_ClientReq: 
             *p++ = 0xBD;
             p = emit32(p, VEX_TRC_JMP_CLIENTREQ); break;
-         case Ijk_Syscall: 
+         case Ijk_Sys_int128:
             *p++ = 0xBD;
-            p = emit32(p, VEX_TRC_JMP_SYSCALL); break;
+            p = emit32(p, VEX_TRC_JMP_SYS_INT128); break;
          case Ijk_Yield: 
             *p++ = 0xBD;
             p = emit32(p, VEX_TRC_JMP_YIELD); break;
@@ -2165,6 +2178,12 @@ Int emit_X86Instr ( UChar* buf, Int nbuf, X86Instr* i )
          case Ijk_TInval:
             *p++ = 0xBD;
             p = emit32(p, VEX_TRC_JMP_TINVAL); break;
+         case Ijk_NoRedir:
+            *p++ = 0xBD;
+            p = emit32(p, VEX_TRC_JMP_NOREDIR); break;
+         case Ijk_Sys_sysenter:
+            *p++ = 0xBD;
+            p = emit32(p, VEX_TRC_JMP_SYS_SYSENTER); break;
          case Ijk_Ret:
 	 case Ijk_Call:
          case Ijk_Boring:
@@ -2176,20 +2195,30 @@ Int emit_X86Instr ( UChar* buf, Int nbuf, X86Instr* i )
 
       /* Get the destination address into %eax */
       if (i->Xin.Goto.dst->tag == Xri_Imm) {
-         /* movl $immediate, %eax ; ret */
+         /* movl $immediate, %eax */
          *p++ = 0xB8;
          p = emit32(p, i->Xin.Goto.dst->Xri.Imm.imm32);
       } else {
          vassert(i->Xin.Goto.dst->tag == Xri_Reg);
-         /* movl %reg, %eax ; ret */
+         /* movl %reg, %eax */
          if (i->Xin.Goto.dst->Xri.Reg.reg != hregX86_EAX()) {
             *p++ = 0x89;
             p = doAMode_R(p, i->Xin.Goto.dst->Xri.Reg.reg, hregX86_EAX());
          }
       }
 
-      /* ret */
-      *p++ = 0xC3;
+      /* Get the dispatcher address into %edx.  This has to happen
+         after the load of %eax since %edx might be carrying the value
+         destined for %eax immediately prior to this Xin_Goto. */
+      vassert(sizeof(UInt) == sizeof(void*));
+      vassert(dispatch != NULL);
+      /* movl $imm32, %edx */
+      *p++ = 0xBA;
+      p = emit32(p, (UInt)Ptr_to_ULong(dispatch));
+
+      /* jmp *%edx */
+      *p++ = 0xFF;
+      *p++ = 0xE2;
 
       /* Fix up the conditional jump, if there was one. */
       if (i->Xin.Goto.cond != Xcc_ALWAYS) {
@@ -2218,7 +2247,7 @@ Int emit_X86Instr ( UChar* buf, Int nbuf, X86Instr* i )
 
       /* Alternative version which works on any x86 variant. */
       /* jmp fwds if !condition */
-      *p++ = 0x70 + (i->Xin.CMov32.cond ^ 1);
+      *p++ = toUChar(0x70 + (i->Xin.CMov32.cond ^ 1));
       *p++ = 0; /* # of bytes in the next bit, which we don't know yet */
       ptmp = p;
 
@@ -2242,7 +2271,7 @@ Int emit_X86Instr ( UChar* buf, Int nbuf, X86Instr* i )
             goto bad;
       }
       /* Fill in the jump offset. */
-      *(ptmp-1) = p - ptmp;
+      *(ptmp-1) = toUChar(p - ptmp);
       goto done;
 
       break;
@@ -2310,27 +2339,28 @@ Int emit_X86Instr ( UChar* buf, Int nbuf, X86Instr* i )
    case Xin_MFence:
       /* see comment in hdefs.h re this insn */
       if (0) vex_printf("EMIT FENCE\n");
-      switch (i->Xin.MFence.subarch) {
-         case VexSubArchX86_sse0:
-            vassert(0); /* awaiting test case */
-            /* lock addl $0,0(%esp) */
-            *p++ = 0xF0; *p++ = 0x83; *p++ = 0x44; 
-            *p++ = 0x24; *p++ = 0x00; *p++ = 0x00;
-            goto done;
-         case VexSubArchX86_sse1:
-            /* sfence */
-            *p++ = 0x0F; *p++ = 0xAE; *p++ = 0xF8;
-            /* lock addl $0,0(%esp) */
-            *p++ = 0xF0; *p++ = 0x83; *p++ = 0x44; 
-            *p++ = 0x24; *p++ = 0x00; *p++ = 0x00;
-            goto done;
-         case VexSubArchX86_sse2:
-            /* mfence */
-            *p++ = 0x0F; *p++ = 0xAE; *p++ = 0xF0;
-            goto done;
-         default: 
-            vpanic("emit_X86Instr:mfence:subarch");
+      if (i->Xin.MFence.hwcaps & (VEX_HWCAPS_X86_SSE3
+                                  |VEX_HWCAPS_X86_SSE2)) {
+         /* mfence */
+         *p++ = 0x0F; *p++ = 0xAE; *p++ = 0xF0;
+         goto done;
       }
+      if (i->Xin.MFence.hwcaps & VEX_HWCAPS_X86_SSE1) {
+         /* sfence */
+         *p++ = 0x0F; *p++ = 0xAE; *p++ = 0xF8;
+         /* lock addl $0,0(%esp) */
+         *p++ = 0xF0; *p++ = 0x83; *p++ = 0x44; 
+         *p++ = 0x24; *p++ = 0x00; *p++ = 0x00;
+         goto done;
+      }
+      if (i->Xin.MFence.hwcaps == 0/*baseline, no SSE*/) {
+         /* lock addl $0,0(%esp) */
+         *p++ = 0xF0; *p++ = 0x83; *p++ = 0x44; 
+         *p++ = 0x24; *p++ = 0x00; *p++ = 0x00;
+         goto done;
+      }
+      vpanic("emit_X86Instr:mfence:hwcaps");
+      /*NOTREACHED*/
       break;
 
    case Xin_Store:
@@ -2632,6 +2662,7 @@ Int emit_X86Instr ( UChar* buf, Int nbuf, X86Instr* i )
          case Xsse_CMPEQF: *p++ = 0xC2; xtra = 0x100; break;
          case Xsse_CMPLTF: *p++ = 0xC2; xtra = 0x101; break;
          case Xsse_CMPLEF: *p++ = 0xC2; xtra = 0x102; break;
+         case Xsse_CMPUNF: *p++ = 0xC2; xtra = 0x103; break;
          default: goto bad;
       }
       p = doAMode_R(p, fake(vregNo(i->Xin.Sse32Fx4.dst)),
@@ -2657,6 +2688,7 @@ Int emit_X86Instr ( UChar* buf, Int nbuf, X86Instr* i )
          case Xsse_CMPEQF: *p++ = 0xC2; xtra = 0x100; break;
          case Xsse_CMPLTF: *p++ = 0xC2; xtra = 0x101; break;
          case Xsse_CMPLEF: *p++ = 0xC2; xtra = 0x102; break;
+         case Xsse_CMPUNF: *p++ = 0xC2; xtra = 0x103; break;
          default: goto bad;
       }
       p = doAMode_R(p, fake(vregNo(i->Xin.Sse64Fx2.dst)),
@@ -2682,6 +2714,7 @@ Int emit_X86Instr ( UChar* buf, Int nbuf, X86Instr* i )
          case Xsse_CMPEQF: *p++ = 0xC2; xtra = 0x100; break;
          case Xsse_CMPLTF: *p++ = 0xC2; xtra = 0x101; break;
          case Xsse_CMPLEF: *p++ = 0xC2; xtra = 0x102; break;
+         case Xsse_CMPUNF: *p++ = 0xC2; xtra = 0x103; break;
          default: goto bad;
       }
       p = doAMode_R(p, fake(vregNo(i->Xin.Sse32FLo.dst)),
@@ -2707,6 +2740,7 @@ Int emit_X86Instr ( UChar* buf, Int nbuf, X86Instr* i )
          case Xsse_CMPEQF: *p++ = 0xC2; xtra = 0x100; break;
          case Xsse_CMPLTF: *p++ = 0xC2; xtra = 0x101; break;
          case Xsse_CMPLEF: *p++ = 0xC2; xtra = 0x102; break;
+         case Xsse_CMPUNF: *p++ = 0xC2; xtra = 0x103; break;
          default: goto bad;
       }
       p = doAMode_R(p, fake(vregNo(i->Xin.Sse64FLo.dst)),
@@ -2809,7 +2843,7 @@ Int emit_X86Instr ( UChar* buf, Int nbuf, X86Instr* i )
    }
 
   bad:
-   ppX86Instr(i);
+   ppX86Instr(i, mode64);
    vpanic("emit_X86Instr");
    /*NOTREACHED*/
    
