@@ -1,14 +1,13 @@
 
 /*--------------------------------------------------------------------*/
-/*--- AMD64/Linux-specific kernel interface.                       ---*/
-/*---                                            vki-amd64-linux.h ---*/
+/*--- AMD64/Linux-specific kernel interface.     vki-amd64-linux.h ---*/
 /*--------------------------------------------------------------------*/
 
 /*
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2005 Julian Seward 
+   Copyright (C) 2000-2006 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -216,6 +215,8 @@ struct vki_sigcontext {
 #define VKI_PROT_WRITE	0x2		/* page can be written */
 #define VKI_PROT_EXEC	0x4		/* page can be executed */
 #define VKI_PROT_NONE	0x0		/* page can not be accessed */
+#define VKI_PROT_GROWSDOWN	0x01000000	/* mprotect flag: extend change to start of growsdown vma */
+#define VKI_PROT_GROWSUP	0x02000000	/* mprotect flag: extend change to end of growsup vma */
 
 #define VKI_MAP_SHARED	0x01		/* Share changes */
 #define VKI_MAP_PRIVATE	0x02		/* Changes are private */
@@ -229,11 +230,13 @@ struct vki_sigcontext {
 
 #define VKI_O_RDONLY	     00
 #define VKI_O_WRONLY	     01
+#define VKI_O_RDWR	     02
 #define VKI_O_CREAT	   0100	/* not fcntl */
 #define VKI_O_EXCL	   0200	/* not fcntl */
 #define VKI_O_TRUNC	  01000	/* not fcntl */
 #define VKI_O_APPEND	  02000
 #define VKI_O_NONBLOCK	  04000
+#define VKI_O_LARGEFILE	0100000
 
 #define VKI_F_DUPFD		0	/* dup */
 #define VKI_F_GETFD		1	/* get close_on_exec */
@@ -430,6 +433,9 @@ struct vki_termios {
 #define VKI_TIOCSPTLCK	_VKI_IOW('T',0x31, int)  /* Lock/unlock Pty */
 
 #define VKI_FIOASYNC	0x5452
+#define VKI_TIOCSERGETLSR   0x5459 /* Get line status register */
+
+#define VKI_TIOCGICOUNT	0x545D	/* read serial port inline interrupt counts */
 
 //----------------------------------------------------------------------
 // From linux-2.6.9/include/asm-x86_64/poll.h
@@ -479,6 +485,8 @@ typedef unsigned long vki_elf_greg_t;
 #define VKI_ELF_NGREG (sizeof (struct vki_user_regs_struct) / sizeof(vki_elf_greg_t))
 typedef vki_elf_greg_t vki_elf_gregset_t[VKI_ELF_NGREG];
 
+typedef struct vki_user_i387_struct vki_elf_fpregset_t;
+
 //----------------------------------------------------------------------
 // From linux-2.6.9/include/asm-x86_64/ucontext.h
 //----------------------------------------------------------------------
