@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2005 Julian Seward 
+   Copyright (C) 2000-2006 Julian Seward 
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -33,8 +33,6 @@
 
    All code is copied verbatim from kernel source files, except that:
    - VKI_/vki_ prefixes are added
-   - a few extra constants are defined (eg. VKI_MAP_NOSYMS);  these are
-     marked 'internal'
    - some extra explanatory comments are included;  they are all within
      "[[ ]]"
    - for some types, we only care about the size;  for a few of them (big
@@ -72,6 +70,8 @@
 #  include "vki_posixtypes-amd64-linux.h"
 #elif defined(VGA_ppc32)
 #  include "vki_posixtypes-ppc32-linux.h"
+#elif defined(VGA_ppc64)
+#  include "vki_posixtypes-ppc64-linux.h"
 #else
 #  error Unknown platform
 #endif
@@ -152,6 +152,8 @@ typedef unsigned int	        vki_uint;
 #  include "vki-amd64-linux.h"
 #elif defined(VGA_ppc32)
 #  include "vki-ppc32-linux.h"
+#elif defined(VGA_ppc64)
+#  include "vki-ppc64-linux.h"
 #else
 #  error Unknown platform
 #endif
@@ -183,58 +185,6 @@ struct vki_sysinfo {
 	char _f[20-2*sizeof(long)-sizeof(int)];	/* Padding: libc5 uses this.. */
 };
 
-//----------------------------------------------------------------------
-// From linux-2.6.8.1/include/linux/byteorder/swab.h
-//----------------------------------------------------------------------
-
-#define ___vki_swab16(x) \
-({ \
-	__vki_u16 __x = (x); \
-	((__vki_u16)( \
-		(((__vki_u16)(__x) & (__vki_u16)0x00ffU) << 8) | \
-		(((__vki_u16)(__x) & (__vki_u16)0xff00U) >> 8) )); \
-})
-
-/*
- * provide defaults when no architecture-specific optimization is detected
- */
-#ifndef __vki_arch__swab16
-#  define __vki_arch__swab16(x) ({ __vki_u16 __tmp = (x) ; ___vki_swab16(__tmp); })
-#endif
-
-// [[Nb: using the non-OPTIMIZE version here -- easier to understand, and
-//   we don't need the optimised version as we use this very rarely]]
-#  define __vki_swab16(x) __vki_fswab16(x)
-
-static __inline__ __attribute_const__ __vki_u16 __vki_fswab16(__vki_u16 x)
-{
-	return __vki_arch__swab16(x);
-}
-
-//----------------------------------------------------------------------
-// From linux-2.6.8.1/include/linux/byteorder/little_endian.h
-//----------------------------------------------------------------------
-
-#ifdef VKI_LITTLE_ENDIAN
-#define __vki_be16_to_cpu(x) __vki_swab16((x))
-#endif // VKI_LITTLE_ENDIAN
-
-//----------------------------------------------------------------------
-// From linux-2.6.8.1/include/linux/byteorder/big_endian.h
-//----------------------------------------------------------------------
-
-#ifdef VKI_BIG_ENDIAN
-#define __vki_be16_to_cpu(x) ((__vki_u16)(x))
-#endif // VKI_BIG_ENDIAN
-
-//----------------------------------------------------------------------
-// From linux-2.6.8.1/include/linux/byteorder/generic.h
-//----------------------------------------------------------------------
-
-#define ___vki_ntohs(x) __vki_be16_to_cpu(x)
-
-#define vki_ntohs(x) ___vki_ntohs(x)
-
 //----------------------------------------------------------------------
 // From linux-2.6.8.1/include/linux/time.h
 //----------------------------------------------------------------------
@@ -433,6 +383,8 @@ typedef struct vki_siginfo {
 	} _sifields;
 } vki_siginfo_t;
 
+#define __VKI_SI_FAULT	0
+
 /*
  * si_code values
  * Digital reserves positive values for kernel-generated signals.
@@ -440,6 +392,49 @@ typedef struct vki_siginfo {
 #define VKI_SI_USER	0		/* sent by kill, sigsend, raise */
 #define VKI_SI_TKILL	-6		/* sent by tkill system call */
 
+/*
+ * SIGILL si_codes
+ */
+#define VKI_ILL_ILLOPC	(__VKI_SI_FAULT|1)	/* illegal opcode */
+#define VKI_ILL_ILLOPN	(__VKI_SI_FAULT|2)	/* illegal operand */
+#define VKI_ILL_ILLADR	(__VKI_SI_FAULT|3)	/* illegal addressing mode */
+#define VKI_ILL_ILLTRP	(__VKI_SI_FAULT|4)	/* illegal trap */
+#define VKI_ILL_PRVOPC	(__VKI_SI_FAULT|5)	/* privileged opcode */
+#define VKI_ILL_PRVREG	(__VKI_SI_FAULT|6)	/* privileged register */
+#define VKI_ILL_COPROC	(__VKI_SI_FAULT|7)	/* coprocessor error */
+#define VKI_ILL_BADSTK	(__VKI_SI_FAULT|8)	/* internal stack error */
+
+/*
+ * SIGFPE si_codes
+ */
+#define VKI_FPE_INTDIV	(__VKI_SI_FAULT|1)	/* integer divide by zero */
+#define VKI_FPE_INTOVF	(__VKI_SI_FAULT|2)	/* integer overflow */
+#define VKI_FPE_FLTDIV	(__VKI_SI_FAULT|3)	/* floating point divide by zero */
+#define VKI_FPE_FLTOVF	(__VKI_SI_FAULT|4)	/* floating point overflow */
+#define VKI_FPE_FLTUND	(__VKI_SI_FAULT|5)	/* floating point underflow */
+#define VKI_FPE_FLTRES	(__VKI_SI_FAULT|6)	/* floating point inexact result */
+#define VKI_FPE_FLTINV	(__VKI_SI_FAULT|7)	/* floating point invalid operation */
+#define VKI_FPE_FLTSUB	(__VKI_SI_FAULT|8)	/* subscript out of range */
+
+/*
+ * SIGSEGV si_codes
+ */
+#define VKI_SEGV_MAPERR	(__VKI_SI_FAULT|1)	/* address not mapped to object */
+#define VKI_SEGV_ACCERR	(__VKI_SI_FAULT|2)	/* invalid permissions for mapped object */
+
+/*
+ * SIGBUS si_codes
+ */
+#define VKI_BUS_ADRALN	(__VKI_SI_FAULT|1)	/* invalid address alignment */
+#define VKI_BUS_ADRERR	(__VKI_SI_FAULT|2)	/* non-existant physical address */
+#define VKI_BUS_OBJERR	(__VKI_SI_FAULT|3)	/* object specific hardware error */
+
+/*
+ * SIGTRAP si_codes
+ */
+#define VKI_TRAP_BRKPT      (__VKI_SI_FAULT|1)  /* process breakpoint */
+#define VKI_TRAP_TRACE      (__VKI_SI_FAULT|2)  /* process trace trap */
+
 /*
  * This works because the alignment is ok on all current architectures
  * but we leave open this being overridden in the future
@@ -477,10 +472,6 @@ typedef struct vki_sigevent {
 #define VKI_SEEK_CUR              1
 #define VKI_SEEK_END              2
 
-// [[Our own additional mmap flags]]
-#define VKI_MAP_NOSYMS  0x40000000      // internal: disable symbol loading
-#define VKI_MAP_CLIENT  0x80000000      // internal: distinguish client mappings
-
 //----------------------------------------------------------------------
 // From linux-2.6.8.1/include/linux/net.h
 //----------------------------------------------------------------------
@@ -1088,9 +1079,12 @@ struct  vki_seminfo {
 //----------------------------------------------------------------------
 
 #define	VKI_EPERM		 1	/* Operation not permitted */
+#define	VKI_ENOENT		 2	/* No such file or directory */
 #define	VKI_ESRCH		 3	/* No such process */
 #define	VKI_EINTR		 4	/* Interrupted system call */
+#define VKI_ENOEXEC              8      /* Exec format error */
 #define	VKI_EBADF		 9	/* Bad file number */
+#define VKI_ECHILD              10      /* No child processes */
 #define VKI_EAGAIN		11	/* Try again */
 #define VKI_EWOULDBLOCK		VKI_EAGAIN
 #define	VKI_ENOMEM		12	/* Out of memory */
@@ -1105,6 +1099,7 @@ struct  vki_seminfo {
 //----------------------------------------------------------------------
 
 #define	VKI_ENOSYS		38	/* Function not implemented */
+#define	VKI_EOVERFLOW		75	/* Value too large for defined data type */
 
 //----------------------------------------------------------------------
 // From linux-2.6.8.1/include/linux/wait.h
@@ -1132,6 +1127,37 @@ struct  vki_seminfo {
 #define VKI_FUTEX_REQUEUE (3)
 #define VKI_FUTEX_CMP_REQUEUE (4)
 
+struct vki_robust_list {
+	struct vki_robust_list __user *next;
+};
+
+struct vki_robust_list_head {
+	/*
+	 * The head of the list. Points back to itself if empty:
+	 */
+	struct vki_robust_list list;
+
+	/*
+	 * This relative offset is set by user-space, it gives the kernel
+	 * the relative position of the futex field to examine. This way
+	 * we keep userspace flexible, to freely shape its data-structure,
+	 * without hardcoding any particular offset into the kernel:
+	 */
+	long futex_offset;
+
+	/*
+	 * The death of the thread may race with userspace setting
+	 * up a lock's links. So to handle this race, userspace first
+	 * sets this field to the address of the to-be-taken lock,
+	 * then does the lock acquire, and then adds itself to the
+	 * list, and then clears this field. Hence the kernel will
+	 * always have full knowledge of all locks that the thread
+	 * _might_ have taken. We check the owner TID in any case,
+	 * so only truly owned locks will be handled.
+	 */
+	struct vki_robust_list __user *list_op_pending;
+};
+
 //----------------------------------------------------------------------
 // From linux-2.6.8.1/include/linux/errno.h
 //----------------------------------------------------------------------
@@ -1142,8 +1168,40 @@ struct  vki_seminfo {
 // From linux-2.6.8.1/include/linux/stat.h
 //----------------------------------------------------------------------
 
+#define VKI_S_IFMT  00170000
+#define VKI_S_IFSOCK 0140000
+#define VKI_S_IFLNK  0120000
+#define VKI_S_IFREG  0100000
+#define VKI_S_IFBLK  0060000
+#define VKI_S_IFDIR  0040000
+#define VKI_S_IFCHR  0020000
+#define VKI_S_IFIFO  0010000
+#define VKI_S_ISUID  0004000
+#define VKI_S_ISGID  0002000
+#define VKI_S_ISVTX  0001000
+
+#define VKI_S_ISLNK(m)	(((m) & VKI_S_IFMT) == VKI_S_IFLNK)
+#define VKI_S_ISREG(m)	(((m) & VKI_S_IFMT) == VKI_S_IFREG)
+#define VKI_S_ISDIR(m)	(((m) & VKI_S_IFMT) == VKI_S_IFDIR)
+#define VKI_S_ISCHR(m)	(((m) & VKI_S_IFMT) == VKI_S_IFCHR)
+#define VKI_S_ISBLK(m)	(((m) & VKI_S_IFMT) == VKI_S_IFBLK)
+#define VKI_S_ISFIFO(m)	(((m) & VKI_S_IFMT) == VKI_S_IFIFO)
+#define VKI_S_ISSOCK(m)	(((m) & VKI_S_IFMT) == VKI_S_IFSOCK)
+
+#define VKI_S_IRWXU 00700
 #define VKI_S_IRUSR 00400
 #define VKI_S_IWUSR 00200
+#define VKI_S_IXUSR 00100
+
+#define VKI_S_IRWXG 00070
+#define VKI_S_IRGRP 00040
+#define VKI_S_IWGRP 00020
+#define VKI_S_IXGRP 00010
+
+#define VKI_S_IRWXO 00007
+#define VKI_S_IROTH 00004
+#define VKI_S_IWOTH 00002
+#define VKI_S_IXOTH 00001
 
 //----------------------------------------------------------------------
 // From linux-2.6.8.1/include/linux/dirent.h
@@ -1303,6 +1361,8 @@ struct vki_shmid_ds {
 	void			*shm_unused3;	/* unused */
 };
 
+#define VKI_SHM_RDONLY  010000  /* read-only access */
+
 #define VKI_SHM_STAT 	13
 #define VKI_SHM_INFO 	14
 
@@ -1592,6 +1652,8 @@ typedef struct vki_sg_scsi_id { /* used by SG_GET_SCSI_ID ioctl() */
                                            if available (struct cdrom_mcn) */
 #define VKI_CDROMVOLREAD	0x5313 /* Get the drive's volume setting
                                           (struct cdrom_volctrl) */
+#define VKI_CDROMREADRAW	0x5314	/* read data in raw mode (2352 Bytes)
+                                           (struct cdrom_read) */
 #define VKI_CDROM_CLEAR_OPTIONS	0x5321  /* Clear behavior options */
 #define VKI_CDROM_DRIVE_STATUS	0x5326  /* Get tray position, etc. */
 
@@ -1798,6 +1860,7 @@ struct vki_hd_geometry {
 };
 
 #define VKI_HDIO_GETGEO		0x0301	/* get device geometry */
+#define VKI_HDIO_GET_DMA	0x030b	/* get use-dma flag */
 #define VKI_HDIO_GET_IDENTITY	0x030d	/* get IDE identification info */
 
 // [[Nb: done like this because the original type is a huge struct that will
@@ -1988,9 +2051,49 @@ typedef __vki_kernel_uid32_t vki_qid_t; /* Type in which we store ids in memory
 // From linux-2.6.9/include/linux/ptrace.h
 //----------------------------------------------------------------------
 
+#define VKI_PTRACE_TRACEME         0
 #define VKI_PTRACE_PEEKTEXT	   1
 #define VKI_PTRACE_PEEKDATA	   2
 #define VKI_PTRACE_PEEKUSR	   3
+#define VKI_PTRACE_POKEUSR	   6
+
+#define VKI_PTRACE_DETACH       0x11
+
+//----------------------------------------------------------------------
+// From linux-2.6.14/include/sound/asound.h
+//----------------------------------------------------------------------
+
+enum {
+	VKI_SNDRV_PCM_IOCTL_HW_FREE = _VKI_IO('A', 0x12),
+	VKI_SNDRV_PCM_IOCTL_HWSYNC = _VKI_IO('A', 0x22),
+	VKI_SNDRV_PCM_IOCTL_PREPARE = _VKI_IO('A', 0x40),
+	VKI_SNDRV_PCM_IOCTL_RESET = _VKI_IO('A', 0x41),
+	VKI_SNDRV_PCM_IOCTL_START = _VKI_IO('A', 0x42),
+	VKI_SNDRV_PCM_IOCTL_DROP = _VKI_IO('A', 0x43),
+	VKI_SNDRV_PCM_IOCTL_DRAIN = _VKI_IO('A', 0x44),
+	VKI_SNDRV_PCM_IOCTL_RESUME = _VKI_IO('A', 0x47),
+	VKI_SNDRV_PCM_IOCTL_XRUN = _VKI_IO('A', 0x48),
+	VKI_SNDRV_PCM_IOCTL_UNLINK = _VKI_IO('A', 0x61),
+};
+
+enum {
+	VKI_SNDRV_TIMER_IOCTL_START = _VKI_IO('T', 0xa0),
+	VKI_SNDRV_TIMER_IOCTL_STOP = _VKI_IO('T', 0xa1),
+	VKI_SNDRV_TIMER_IOCTL_CONTINUE = _VKI_IO('T', 0xa2),
+	VKI_SNDRV_TIMER_IOCTL_PAUSE = _VKI_IO('T', 0xa3),
+};
+
+//----------------------------------------------------------------------
+// From linux-2.6.15.4/include/linux/serial.h
+//----------------------------------------------------------------------
+
+struct vki_serial_icounter_struct {
+	int cts, dsr, rng, dcd;
+	int rx, tx;
+	int frame, overrun, parity, brk;
+	int buf_overrun;
+	int reserved[9];
+};
 
 #endif // __VKI_LINUX_H
 
