@@ -31,8 +31,8 @@ int main()
 	if (ret != -1 || errno != EINTR) {
 		printf("FAILED: expected nanosleep to be interrupted\n");
 	} else {
-		VALGRIND_CHECK_DEFINED(rem);
-		printf("PASSED\n"); /* assuming CHECK_DEFINED doesn't print anything */
+		VALGRIND_CHECK_VALUE_IS_DEFINED(rem);
+		printf("PASSED\n"); /* assuming CHECK_VALUE_IS_DEFINED doesn't print anything */
 	}
 
 	return 0;
