@@ -1,5 +1,8 @@
 #include <signal.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <sys/types.h>
+#include <unistd.h>
 
 static char *deep;
 
